-- This file is a part of Avoision.
-- 
-- Copyright (C) 2011 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("Avoision/Common.lua")
require("Effects/Ghost.lua")

local playerVelocity = 0.6 / 1000
local playerWidth = 1 / 30
local playerHeight = 1 / 30

local playerGhostPeriod = 750
local playerGhostScaleMax = 6

Player = { }

function Player.new()
    local player = Entity.new(0, 0, 0, playerWidth, playerHeight)

    player.color = Common.playerColor
    player.mesh = Common.squareMesh
    player.group = Common.group.player
    player.v = { 0, 0, 0, 0 }

    player.elements:add(Element.Image.new("Images/Block.png", 0, 0, 0, 1, 1, 0))

    function player.setMovingUpState(self, moving)
        if moving then
            self.v[1] = 1
        else
            self.v[1] = 0
        end
    end
    
    function player.setMovingDownState(self, moving)
        if moving then
            self.v[2] = 1
        else
            self.v[2] = 0
        end
    end
    
    function player.setMovingLeftState(self, moving)
        if moving then
            self.v[3] = 1
        else
            self.v[3] = 0
        end
    end
    
    function player.setMovingRightState(self, moving)
        if moving then
            self.v[4] = 1
        else
            self.v[4] = 0
        end
    end
    
    function player.clearMovingStates(self)
        self.v[1] = 0
        self.v[2] = 0
        self.v[3] = 0
        self.v[4] = 0
    end
    
    function player.update(self, ms)
        local directionX = self.v[4] - self.v[3]
        local directionY = self.v[1] - self.v[2]
    
        if directionX ~= 0 or directionY ~= 0 then
            local direction = arctan(directionY, directionX)
        
            self.x = self.x + playerVelocity * ms * cos(direction)
            self.y = self.y + playerVelocity * ms * sin(direction)
    
            if abs(self.x) + self.width / 2 > 0.5 then
                if self.x > 0 then
                    self.x = 0.5 - self.width / 2
                else
                    self.x = -0.5 + self.width / 2
                end
            end
    
            if abs(self.y) + self.height / 2 > 0.5 then
                if self.y > 0 then
                    self.y = 0.5 - self.height / 2
                else
                    self.y = -0.5 + self.height / 2
                end
            end
        end
    end

    function player.createGhost(self)
        return Ghost.new(self, playerGhostPeriod, playerGhostScaleMax)
    end

    return player
end

