-- This file is a part of Avoision.
-- 
-- Copyright (C) 2012 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("Utility/SortedList.lua")
require("Avoision/Common.lua")
require("Settings/HighScoreDefaultData.lua")
include("Settings/HighScoreData.lua")

HighScore = { }

HighScore.count = 4

function HighScore.compare(a, b)
    local v = 0

    if a.score > b.score then
        v = -1
    elseif a.score < b.score then
        v = 1
    end

    return v
end

function HighScore.new(difficulty, name, score)
    return {
        difficulty = difficulty,
        name = name,
        score = score,
    }
end

function HighScore.getLowestScore(difficulty)
    local list = highScores[difficulty]
    local lowestScore = 0

    if isTable(list) then
        local lastScore = list:getLast()

        if not isNil(lastScore) then
            lowestScore = lastScore.score
        end
    end

    return lowestScore
end

function HighScore.insert(highScore)
    local list = highScores[highScore.difficulty]

    if isTable(list) then
        list:insert(highScore)
        list:truncate(HighScore.count)
    end
end

function HighScore.save()
    File.createDirectory("Settings")

    local file = File.openWrite("Settings/HighScoreData.lua")
    local difficulty = Common.difficultyLevel.Easy

    while not isNil(highScores[difficulty]) do
        local i = 1

        highScores[difficulty]:forEach(function(highScore)
            file:write(String.format("highScoreData[%d][%d] = { %q, %d }", difficulty, i, highScore.name, highScore.score))
            i = i + 1
        end)

        difficulty = difficulty + 1
    end

    file:close()
end

-- Set up highScores lists
highScores = {
    [Common.difficultyLevel.Easy] = SortedList.new(HighScore.compare),
    [Common.difficultyLevel.Normal] = SortedList.new(HighScore.compare),
    [Common.difficultyLevel.Hard] = SortedList.new(HighScore.compare),
    [Common.difficultyLevel.Extreme] = SortedList.new(HighScore.compare),
}

-- Load high score data
local i = 1
local difficulty = Common.difficultyLevel.Easy

while not isNil(highScoreData[i]) do
    local highScores = highScoreData[i]
    local j = 1

    while not isNil(highScores[j]) do
        local highScore = highScores[j]

        HighScore.insert(HighScore.new(difficulty, highScore[1], highScore[2]))

        j = j + 1
    end

    i = i + 1
    difficulty = difficulty + 1
end

