-- This file is a part of Avoision.
-- 
-- Copyright (C) 2010 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

NumberLabel = { }

function NumberLabel.new(table, key, x, y, z, width, height, alignment, valueChanged)
    local numberLabel = Entity.new(x, y, z, width, height)

    numberLabel.table = table
    numberLabel.key = key
    numberLabel.lastValue = 0

    numberLabel.textElement = numberLabel.elements:add(Element.Text.new("0"))

    if isString(alignment) then
        numberLabel.textElement.alignment = alignment
    end

    if isFunction(valueChanged) then
        numberLabel.valueChanged = valueChanged
    end

    function numberLabel.update(self)
        local value = self.table[self.key]
        
        if value ~= self.lastValue then
            -- TODO: String formatting
            self.textElement.text = "" .. value
            self.lastValue = value

            if isFunction(self.valueChanged) then
                self:valueChanged(value)
            end
        end
    end

    function numberLabel.getPosition(self)
        return self.x, self.y
    end

    function numberLabel.setPosition(self, x, y)
        self.x = x
        self.y = y
    end

    function numberLabel.getSize(self)
        return #self.textElement.text * self.width, self.height
    end

    function numberLabel.setTextSize(self, width, height)
        self.width = width
        self.height = height
    end

    return numberLabel
end

